# 2021-6-30：Write by lyc at，基础功能封装

# 从Python SDK导入SMS配置管理模块以及安全认证模块
from baidubce.bce_client_configuration import BceClientConfiguration
from baidubce.auth.bce_credentials import BceCredentials

# 导入SMS相关模块
import baidubce.services.sms.sms_client as sms
import baidubce.exception as ex


class SMS():
    '''百度云SMS短信服务 Python3 SDK封装'''
    def __init__(self, key):
        self.ak = key['Access_Key']
        self.sk = key['Secret_Access_Key']
        self.endpoint = key['SMS']['Endpoint']
        self.signature_id = key['SMS']['Signature_ID']
        self.template_id = key['SMS']['Template_ID']

        try:
            # 创建BceClientConfiguration
            self.config = BceClientConfiguration(credentials=BceCredentials(
                self.ak, self.sk),
                                                 endpoint=self.endpoint)
            # 新建SmsClient
            self.sms_client = sms.SmsClient(self.config)
        except Exception as err:
            raise Exception(err)  # 连接失败主动抛出异常

    def sendMessage(self, mobile, content):
        '''
        发送短信
        :param mobile: 手机号码
        :param content: dict类型，字典内变量value 类型必须 str，变量名称对应模板变量传入，比如{'code': '1236', 'time': '5'}
        :return: response 响应对象
        '''
        try:
            response = self.sms_client.send_message(self.signature_id,
                                                    self.template_id,
                                                    mobile=mobile,
                                                    content_var_dict=content)
            return response
        except ex.BceHttpClientError as e:
            if isinstance(e.last_error, ex.BceServerError):
                print(
                    'send request failed. Response %s, code: %s, request_id: %s'
                    % (e.last_error.status_code, e.last_error.code,
                       e.last_error.request_id))
            else:
                print('send request failed. Unknown exception: %s' % e)



import operator
from ucloud.core import auth

# Write by lyc at 2020-4-29
# [UCloud api签名算法](https://docs.ucloud.cn/api/summary/signature)

def verify_ac(public_key, private_key, parm):
    '''
    计算签名 signature
    :param public_key: 公钥
    :param private_key: 私钥
    :param parm: 按照key升序排列后的请求参数字典
    :return: signature 签名值
    '''
    # 升序排列dict
    parm = dict(sorted(parm.items(), key=operator.itemgetter(0), reverse=False))
    cred = auth.Credential(
        "%s"%public_key,
        "%s"%private_key,
    )
    return cred.verify_ac(parm)









import datetime


# 假设此刻时间是 2021-07-07 17:04:15.889853
def getStrfTime(diff_day=0, timeformat_type='day'):
    """获取格式化的时间戳

    Args:
        diff_day (int, optional): 间隔时间差天数. Defaults to 0.
        timeformat_type (str, optional): 要返回的时间戳格式. Defaults to 'day'.
                        可选参数：(
                        'year' 2021 | 
                        'month' 202107 | 
                        'month_num' 7 | 
                        'day' 20210707 | 
                        'hour' 2021070717 | 
                        'min' 202107071704 |
                        'second' 20210707170415)

    Returns:
        [str]: 返回字符串类型的时间戳
    """
    now = datetime.datetime.now()
    if timeformat_type == 'year':
        timeformat_x = (now + datetime.timedelta(days=diff_day)).strftime("%Y")
    elif timeformat_type == 'month':
        timeformat_x = (now +
                        datetime.timedelta(days=diff_day)).strftime("%Y%m")
    elif timeformat_type == 'month_num':
        timeformat_x = str(
            int((now + datetime.timedelta(days=diff_day)).strftime("%m")))
    elif timeformat_type == 'day':
        timeformat_x = (now +
                        datetime.timedelta(days=diff_day)).strftime("%Y%m%d")
    elif timeformat_type == 'hour':
        timeformat_x = (now +
                        datetime.timedelta(days=diff_day)).strftime("%Y%m%d%H")
    elif timeformat_type == 'min':
        timeformat_x = (
            now + datetime.timedelta(days=diff_day)).strftime("%Y%m%d%H%M")
    elif timeformat_type == 'second':
        timeformat_x = (
            now + datetime.timedelta(days=diff_day)).strftime("%Y%m%d%H%M%S")

    return timeformat_x







# V2021-6-25：Wriet by lyc at，基础功能封装
# V2021-6-30：Update by lyc at，注释优化，增加上传接口类

import os

# 从Python SDK导入BOS配置管理模块以及安全认证模块
from baidubce.bce_client_configuration import BceClientConfiguration
from baidubce.auth.bce_credentials import BceCredentials

# 导入BOS相关模块
from baidubce import exception
from baidubce.services import bos
from baidubce.services.bos import canned_acl
from baidubce.services.bos.bos_client import BosClient


def md5file(file_name):
    """content_md5计算方法是对数据执行md5算法获取128位二进制数据，再进行base64编码

    Args:
        file_name ([str]): [文件绝对路径]

    Returns:
        [str]: [base64编码后的文件md5值]
    """
    import io
    import hashlib
    import base64

    # print(file_name)
    buf_size = 8192
    fp = open(file_name, 'rb')
    md5 = hashlib.md5()
    while True:
        bytes_to_read = buf_size
        buf = fp.read(bytes_to_read)
        if not buf:
            break
        md5.update(buf)
    content_md5 = base64.standard_b64encode(md5.digest())
    return content_md5


class BOS():
    """[百度云BOS对象存储  Python3 SDK封装]"""
    def __init__(self, key):
        self.ak = key['Access_Key']
        self.sk = key['Secret_Access_Key']
        self.endpoint = key['BOS']['Endpoint']
        self.bucket_name = key['BOS']['Bucket']
        try:
            # 创建BceClientConfiguration
            self.config = BceClientConfiguration(credentials=BceCredentials(
                self.ak, self.sk),
                                                 endpoint=self.endpoint)
            # 新建BosClient
            self.bos_client = BosClient(self.config)
        except Exception as err:
            raise Exception(err)  # 连接失败主动抛出异常

    def list_Objects(self, prefix='', marker='None'):
        """[分页获取所有Object]

        Args:
            prefix (str, optional): [过滤指定前缀的object key返回]. Defaults to ''.
            marker (str, optional): [本次查询的起点]. Defaults to 'None'.

        Yields:
            [str]: [一次返回一个 object_key（生成器）]
        """
        isTruncated = True
        max_keys = 500  # 限定此次返回object的最大数，此数值不能超过1000，如果不设定，默认为1000。
        marker = None
        while isTruncated:
            response = self.bos_client.list_objects(self.bucket_name,
                                                    prefix=prefix,
                                                    max_keys=max_keys,
                                                    marker=marker)
            for obj in response.contents:
                yield obj.key
            isTruncated = response.is_truncated
            marker = getattr(response, 'next_marker', None)

    def put_Object(self, file_name, object_key):
        """[简单上传：file_name < 5G]

        Args:
            file_name ([str]): [本地要上传文件的绝对路径]
            object_key ([str]): [文件在对象存储中的名称，文件名可以包含'/'表示目录]

        Returns:
            [int]: [200-成功;!=200 失败]
        """
        try:
            # 从文件中上传的Object
            content_md5 = md5file(file_name).decode('utf-8')  # 本地 content_md5
            response = self.bos_client.put_object_from_file(
                self.bucket_name, object_key, file_name)
            if content_md5 == response.metadata.content_md5:
                return 200
            else:
                return -1
        except Exception as err:
            print(err)

    def put_Object_Part(self, file_name, object_key):
        """[分块上传：file_name > 5G]

        Args:
            file_name ([str]): [本地要上传文件的绝对路径]
            object_key ([str]): [文件在对象存储中的名称，文件名可以包含'/'表示目录]

        Returns:
            [int]: [200-成功;!=200 失败]
        """
        try:
            raise Exception("分块上传程序未完成")
            # https://cloud.baidu.com/doc/BOS/s/sjwvyrg3l#%E5%88%86%E5%9D%97%E4%B8%8A%E4%BC%A0
        except Exception as err:
            print(err)  # 异常处理

    def upload_File(self, file_name, object_key):
        """[上传文件接口类，自动识别上传文件来选择普通上传方法或分块上传方法]

        Args:
            file_name ([str]): [本地要上传文件的绝对路径]
            object_key ([str]): [文件在对象存储中的名称，文件名可以包含'/'表示目录]

        Returns:
            [int]: [200-成功;!=200 失败]
        """
        file_size = os.path.getsize(file_name)
        n = 1024  # count, 10count = 10M；这里我认为大于1024M的是大文件，采用分块上传方法
        size_threshold = n * 1024 * 1024  # 阈值：n=100 即 100M=104857600bytes
        if file_size <= size_threshold:  # 文件大小阈值判断
            return self.put_Object(file_name, object_key)
        else:
            return self.put_Object_Part(file_name, object_key)

    def delete_Object(self, object_key):
        """[删除单个/删除批量 Object文件]

        Args:
            object_key ([str]): [对象存储中的文件名，同时支持传入str或list类型]

        Returns:
            [int]: [200-成功;!=200 失败]
        """
        try:
            if isinstance(object_key, str):
                self.bos_client.delete_object(self.bucket_name,
                                              object_key)  # 删除单个文件
            elif isinstance(object_key, list):
                self.bos_client.delete_multiple_objects(
                    self.bucket_name, object_key)  # 批量删除列表内的文件
            return 200
        except Exception as err:
            print(err)  # 异常处理

    def head_Object(self, object_key):
        """[获取Object元数据信息（大小、最后更新时间等）]

        Args:
            object_key ([str]): [对象存储中的文件名]

        Returns:
            [response.metadata]: [数据元对象]
        """
        try:
            response = self.bos_client.get_object_meta_data(
                self.bucket_name, object_key)
            # print("Get meta:{}", response.metadata)
            return response.metadata

        except exception.BceError as err:
            print(err)  # 异常处理

    def get_Object(self, object_key, save_dir):
        """[简单下载：下载Object到本地文件]

        Args:
            object_key ([str]): [对象存储中的文件名]
            save_dir ([str]): [保存到本地目录]

        Returns:
            [int]: [200-成功;!=200 失败]
        """
        file_name = save_dir + os.sep + object_key.replace(
            '/', os.sep)  # 下载文件本地绝对路径拼接
        os.makedirs(os.path.dirname(file_name), exist_ok=True)  # 创建文件目录
        try:
            response = self.bos_client.get_object_to_file(
                self.bucket_name, object_key, file_name)
            if response.metadata.content_md5 == md5file(file_name).decode(
                    'utf-8'):
                return 200
            else:
                return -1

        except Exception as err:
            print(err)  # 异常处理



from ftplib import FTP


class FTP_Tools():
    """[ftp简单方法封装]
    """
    def __init__(self, host, username, password, port=21):
        self.host = host
        self.port = port
        self.username = username
        self.password = password


    def connect(self):
        """[建立ftp连接]

        Returns:
            [type]: [description]

        Return:
            [obj]: ftp_conn 连接对象
        """
        try:
            ftp_conn = FTP()
            # ftp.set_debuglevel(2)
            ftp_conn.connect(self.host, int(self.port))
            ftp_conn.login(self.username, self.password)
            return ftp_conn
        
        except Exception as err:
            print(err) 


    def upload_file(self, localfile, remotefile, bufsize=1024):
        """[上传文件]

        Args:
            localfile ([str]): [本地文件绝对路径]
            remotefile ([str]): [目标文件绝对路径]
            bufsize (int, optional): [读取文件buff缓存大小]. Defaults to 1024.
        
        Return:
            None
        """

        try:
            ftp_conn=self.connect()
            fp = open(localfile, 'rb')
            ftp_conn.storbinary('STOR ' + remotefile, fp, bufsize)
            ftp_conn.set_debuglevel(0)
            fp.close()
            return

        except Exception as err:
            print(err)


    def download_file(self, remotefile, localfile, bufsize=4096):
        """[下载文件]

        Args:
            remotefile ([str]): [目标文件绝对路径]
            localfile ([str]): [本地文件绝对路径]
            bufsize (int, optional): [读取文件buff缓存大小]. Defaults to 4096.

        Returns:
            None
        """
        try:
            ftp_conn=self.connect()
            fp = open(localfile, 'wb')
            ftp_conn.retrbinary('RETR ' + remotefile, fp.write, bufsize)
            ftp_conn.set_debuglevel(0)
            fp.close()
            return

        except Exception as err:
            print(err)



    def check_remote_dir(self, remotedir):
        """[检查远端ftp目录，不存在则创建]

        Args:
            remotedir ([str]): [远端ftp根目录下的目录]

        Returns:
            None
        """
        
        try:
            ftp_conn=self.connect()
            remotepath_list = remotedir.split('/')
            dir = ''
            index = 1
            for i in remotepath_list:
                if index < len(remotepath_list):
                    dir += i+ '/'
                    if remotepath_list[index] not in ftp_conn.nlst(dir):
                        ftp_conn.mkd(dir + remotepath_list[index])
                    index += 1
            return

        except Exception as err:
            print(err)






import json, requests
from tools_lyc7456.uCloudapi.signature import verify_ac

# Write by lyc at 2020-12-4
# Update by lyc at 2021-10-4：优化成包
# Update by lyc at 2021-11-9：增加方法 createBandwidthPackage()

class UVPC():
    '''优刻得UVPC封装'''
    def __init__(self, key):
        self.PublicKey = key['PublicKey']           # 公钥
        self.PrivateKey = key['PrivateKey']         # 私钥
        self.api_url = key['api_url']               # api接口url
        self.project_id = key['project_id']         # 项目id
        self.region = key['region']                 # 地域
        # uvpc
        self.vpc_id = key['VPC']['vpc_id']          # VPC id
        self.subnet_id = key['VPC']['subnet_id']    # 子网 id
        self.bandwidth = key['VPC']['bandwidth']    # EIP带宽值
        self.paymode = key['VPC']['paymode']        # EIP计费模式
        self.chargetype = key['VPC']['chargetype']  # EIP付费模式
        self.fw_id = key['VPC']['fw_id']            # 防火墙 id


    def allocateEip(self, operatorname='Bgp'):
        """[申请弹性IP - AllocateEIP]https://docs.ucloud.cn/api/unet-api/allocate_eip

        Args:
            operatorname (bool, optional): [弹性IP线路]. "International" 国际线路；"Bgp" 国内IP。默认为 "Bgp".

        Returns:
            [str]: [response]
        """
        try:
            params = {
                'Action': 'AllocateEIP',            # 接口名称
                'Region': self.region,              # 地区
                'OperatorName': operatorname,
                'Bandwidth': self.bandwidth,
                'PayMode': self.paymode,
                'ChargeType': self.chargetype,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err


    def releaseEip(self, eip_id):
        """[释放弹性IP - ReleaseEIP]https://docs.ucloud.cn/api/unet-api/release_eip

        Args:
            eip_id (string): [弹性ip资源id].

        Returns:
            [str]: [response]
        """
        try:
            params = {
                'Action': 'ReleaseEIP',         # 接口名称
                'Region': self.region,
                'EIPId': eip_id,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string
        
        except Exception as err:
            # print(err)
            return err


    def bindEip(self, eip_id, resource_id, resource_type='uhost'):
        """[绑定弹性IP - BindEIP]https://docs.ucloud.cn/api/unet-api/bind_eip

        Args:
            eip_id ([string]): [弹性ip资源id]
            resource_id ([string]): [弹性IP请求绑定的资源id]
            resource_type (str, optional): [弹性IP请求绑定的资源类型]. 枚举值为: uhost: 云主机; ulb, 负载均衡器; uni：虚拟网卡; 默认值 'uhost'.

        Returns:
            [str]: [response]
        """
        try:
            params = {
                'Action': 'BindEIP',                # 接口名称
                'Region': self.region,
                'EIPId': eip_id,
                'ResourceType': resource_type,
                'ResourceId': resource_id,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err



    def unbindEip(self, eip_id, resource_id, resource_type='uhost'):
        """[解绑弹性IP - UnBindEIP]https://docs.ucloud.cn/api/unet-api/un_bind_eip

        Args:
            eip_id ([string]): [弹性ip资源id]
            resource_id ([string]): [弹性IP请求绑定的资源id]
            resource_type (str, optional): [弹性IP请求绑定的资源类型]. 枚举值为: uhost: 云主机; ulb, 负载均衡器; uni：虚拟网卡; 默认值 'uhost'.

        Returns:
            [str]: [response]
        """
        try:
            params = {
                'Action': 'UnBindEIP',      # 接口名称
                'Region': self.region,
                'EIPId': eip_id,
                'ResourceId': resource_id,
                'ResourceType': resource_type,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err



    def describeEIP(self, eip_id='', eip_ip=''):
        """[获取弹性IP信息 - DescribeEIP]https://docs.ucloud.cn/api/unet-api/describe_eip
        
        以下参数有且只能二选一：
        Args:
            eip_id ([string]): [弹性ip资源id]
            eip_ip ([string]): [弹性ip公网IP地址]

        Returns:
            [str]: [response]
        """
        try:
            if len(eip_id) > 0:
                params = {
                    'Action': 'DescribeEIP',      # 接口名称
                    'Region': self.region,
                    'EIPIds.0': eip_id,
                    'ProjectId': self.project_id,
                    'PublicKey': self.PublicKey,
                }
            elif len(eip_ip) > 0:
                params = {
                    'Action': 'DescribeEIP',      # 接口名称
                    'Region': self.region,
                    'IPs.0': eip_ip,
                    'ProjectId': self.project_id,
                    'PublicKey': self.PublicKey,
                }

            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err



    def createNetworkInterface(self):
        """[创建虚拟网卡]

        Returns:
            [str]: [response]
        """
        try:
            params = {
                'Action': 'CreateNetworkInterface',  # 接口名称
                'Region': self.region,
                'VPCId': self.vpc_id,
                'SubnetId': self.subnet_id,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err



    def deleteNetworkInterface(self, uni_id):
        """[删除虚拟网卡]

        Args:
            uni_id ([string]): [虚拟网卡id]

        Returns:
            [dict]: [response]
        """
        try:
            params = {
                'Action': 'DeleteNetworkInterface',  # 接口名称
                'Region': self.region,
                'InterfaceId': uni_id,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err


    def grantFirewall(self, uni_id, resource_type='uhost'):
        """[应用防火墙 - GrantFirewall]https://docs.ucloud.cn/api/unet-api/grant_firewall

        Args:
            uni_id ([string]): [虚拟网卡资源ID]
            resource_type (str, optional): [弹性IP请求绑定的资源类型]. 枚举值为: uhost: 云主机; ulb, 负载均衡器; uni：虚拟网卡; 默认值 'uhost'.

        Returns:
            [dict]: [response]
        """
        try:
            params = {
                'Action': 'GrantFirewall',  # 接口名称
                'Region': self.region,
                'FWId': self.fw_id,
                'ResourceType': resource_type,
                'ResourceId': uni_id,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err



    def attachNetworkInterface(self, uni_id, uhost_id):
        """[虚拟网卡绑定云主机]

        Args:
            uni_id ([string]): [虚拟网卡id]
            uhost_id ([string]): [云主机资源id]

        Returns:
            [dict]: [response]
        """
        try:
            params = {
                'Action': 'AttachNetworkInterface',  # 接口名称
                'Region': self.region,
                'InterfaceId': uni_id,
                'InstanceId': uhost_id,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err



    def describeNetworkInterface(self, uhost_id):
        '''获取网卡绑定信息'''
        try:
            params = {
                'Action': 'DescribeNetworkInterface',  # 接口名称
                'Region': self.region,
                'Limit': 1000,  # 1000个网卡内可以获取到，1000个以上需要重写递归循环
                'Offset': 0,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err



    def detachNetworkInterface(self, uni_id, uhost_id):
        """[虚拟网卡解绑云主机]

        Args:
            uni_id ([string]): [虚拟网卡id]
            uhost_id ([string]): [云主机资源id]

        Returns:
            [dict]: [response]
        """
        try:
            params = {
                'Action': 'DetachNetworkInterface',  # 接口名称
                'Region': self.region,
                'InterfaceId': uni_id,
                'InstanceId': uhost_id,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err



    def createBandwidthPackage(self, eip_id, bandwidth_package_value, bandwidth_package_timerange):
        """[创建带宽包 - CreateBandwidthPackage]https://docs.ucloud.cn/api/unet-api/describe_eip
        
        Args:
            eip_id ([string]): [所绑定弹性IP的资源ID]
            bandwidth_package_value ([int]): [带宽包大小(单位Mbps), 取值范围[2,800] (最大值受地域限制)]
            bandwidth_package_timerange([int]): 带宽包有效时长, 取值范围为大于0的整数, 即该带宽包在EnableTime到 EnableTime+TimeRange时间段内生效

        Returns:
            [str]: [response]
        """
        try:
            params = {
                'Action': 'CreateBandwidthPackage',
                'Region': self.region,
                'Bandwidth': bandwidth_package_value,
                'EIPId': eip_id,
                'TimeRange': bandwidth_package_timerange,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err



    def modifyUPathBandwidth(self, upath_id, upath_bandwidth):
        """[修改加速线路带宽 - ModifyUPathBandwidth]https://docs.ucloud.cn/api/pathx-api/modify_u_path_bandwidth
        
        以下参数有且只能二选一：
        Args:
            upath_id ([string]): [UPath 加速线路实例Id]
            upath_bandwidth([int]): 带宽,单位Mbps 区间[1,800]。

        Returns:
            [str]: [response]
        """
        try:
            params = {
                'Action': 'ModifyUPathBandwidth',
                'UPathId': upath_id,
                'Bandwidth': upath_bandwidth,
                'PublicKey': self.PublicKey,
                'ProjectId': self.project_id,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err






import base64
import json
from operator import le
import requests
from tools_lyc7456.uCloudapi.signature import verify_ac

# Write by lyc at 2020-12-4
# Update by lyc at 2021-2-4：增加绑定云主机告警模板id
# Update by lyc at 2021-10-4：优化成包


class UHOST():
    '''优刻得uhost云主机 api封装'''
    def __init__(self, key):
        self.PublicKey = key['PublicKey']           # 公钥
        self.PrivateKey = key['PrivateKey']         # 私钥
        self.api_url = key['api_url']               # api接口url
        self.project_id = key['project_id']         # 项目id
        self.region = key['region']                 # 地域
        self.zone = key['zone']                     # 可用区
        # uhost
        self.name = key['UHOST']['name']                # 云主机名称
        self.image_id = key['UHOST']['image_id']        # 镜像id
        self.password = key['UHOST']['password']        # 密码
        self.cpu = key['UHOST']['cpu']                  # 虚拟CPU核数
        self.memory = key['UHOST']['memory']            # 虚拟内存大小
        self.disk0_size = key['UHOST']['disk0_size']    # 系统盘大小
        self.chargetype = key['UHOST']['chargetype']    # 计费模式
        self.tag = key['UHOST']['tag']                  # 业务组
        self.alarm_template_id = key['UHOST']['alarm_template_id']  # 告警模板id
        # uvpc
        self.bandwidth = key['VPC']['bandwidth']          # 带宽
        self.paymode = key['VPC']['paymode']              # 带宽付费类型
        self.fw_id = key['VPC']['fw_id']                  # 防火墙id



    def describeUHostInstance(self, uhost_id):
        """[获取主机信息 - DescribeUHostInstance]https://docs.ucloud.cn/api/uhost-api/describe_uhost_instance

        Args:
            uhost_id ([string]): [UHost实例ID]

        Returns:
            [str]: [response]
        """
        try:
            params = {
                'Action': 'DescribeUHostInstance',      # 接口名称
                'Region': self.region,
                'Zone': self.zone,
                'UHostIds.0': uhost_id,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err



    def createUHostInstance(self, disks_0_type='LOCAL_NORMAL', maxcount=1, operatorname='Bgp', features_uni=True):
        """[创建云主机 - CreateUHostInstance]https://docs.ucloud.cn/api/uhost-api/create_uhost_instance

        Args:
            disks_0_type (str, optional): [系统盘类型]. Defaults to 'LOCAL_NORMAL'. 磁盘类型参考：https://docs.ucloud.cn/api/uhost-api/disk_type
            maxcount (int, optional): [本次最大创建主机数量]. Defaults to 1.
            operatorname (str, optional): [弹性IP线路]. "International" 国际线路；"Bgp" 国内IP。默认为 "Bgp".
            features_uni (bool): [弹性IP线路]. 弹性网卡特性。开启了弹性网卡权限位，此特性才生效。默认 True 开启。

        Returns:
            [type]: [description]
        """
        try:
            self.password = (base64.b64encode(self.password.encode("utf-8"))).decode('utf-8')   # 云主机密码使用base64进行编码
            params = {
                'Action': 'CreateUHostInstance',      # 接口名称
                'Region': self.region,
                'Zone': self.zone,
                'ImageId': self.image_id,
                'Password': self.password,
                'Disks.0.Type': disks_0_type,
                'Disks.0.IsBoot': 'True',
                'Disks.0.Size': self.disk0_size,
                'LoginMode': 'Password',
                'Name': self.name,
                'Tag': self.tag,
                'ChargeType': self.chargetype,
                'CPU': self.cpu,
                'Memory': self.memory,
                'SecurityGroupId': self.fw_id,
                'MaxCount': maxcount,
                'NetworkInterface.0.EIP.Bandwidth': self.bandwidth,
                'NetworkInterface.0.EIP.PayMode': self.paymode,
                'NetworkInterface.0.EIP.OperatorName': operatorname,
                'AlarmTemplateId': self.alarm_template_id,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
                'Features.UNI':  features_uni,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err



    def stopUHostInstance(self, uhost_id):
        """[关闭主机 - StopUHostInstance]https://docs.ucloud.cn/api/uhost-api/stop_uhost_instance

        Args:
            uhost_id ([string]): [UHost实例ID]

        Returns:
            [str]: [response]
        """
        try:
            params = {
                'Action': 'StopUHostInstance',      # 接口名称
                'Region': self.region,
                'Zone': self.zone,
                'UHostId': uhost_id,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err



    def terminateUHostInstance(self, uhost_id):
        """[删除云主机 - TerminateUHostInstance]https://docs.ucloud.cn/api/uhost-api/terminate_uhost_instance

        Args:
            uhost_id ([string]): [UHost实例ID]

        Returns:
            [str]: [response]
        """
        try:
            params = {
                'Action': 'TerminateUHostInstance',      # 接口名称
                'Region': self.region,
                'UHostId': uhost_id,
                'ReleaseEIP': True,
                'ReleaseUDisk': True,
                'ProjectId': self.project_id,
                'PublicKey': self.PublicKey,
            }
            # 签名
            signature = verify_ac(self.PublicKey, self.PrivateKey, params)
            params['Signature'] = signature

            # POST请求接口
            headers = {'Content-Type': 'application/json'}
            response = requests.post(url=self.api_url, headers=headers, data=json.dumps(params))
            resp_string = response.content.decode('utf-8')

            return resp_string

        except Exception as err:
            # print(err)
            return err





import os


def scanDir(targetDir):
    """[扫描目标目录] Write by lyc 2021-5-19

    Args:
        targetDir ([str]): [目标目录]

    Yields:
        [dic]: [返回文件的相对路径和绝对路径] 
                file_info = {
                'abs_file_path': ,      # 文件的绝对路径
                'rel_file_path': ,      # 文件的相对路径
                }
    """
    relativePath = targetDir.rsplit(os.sep, 1)[0] + os.sep  # 目标的相对路径
    relativeDir = targetDir.rsplit(os.sep, 1)[-1]  # 目标的相对路径下的第一级目录

    for (dirpath, dirnames, filenames) in os.walk(targetDir):
        for fn in filenames:
            # 把 dirpath 和 每个文件名拼接起来 就是全路径
            abs_file_path = os.path.join(dirpath, fn)  # 文件的绝对路径
            rel_file_path = abs_file_path.replace(relativePath, '')  # 文件的相对路径
            file_info = {
                'abs_file_path': abs_file_path,
                'rel_file_path': rel_file_path,
            }
            yield file_info



import json
from tencentcloud.common import credential
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
from tencentcloud.vpc.v20170312 import vpc_client, models

# Write by lyc at 2021-10-9

class VPC():
    """[腾讯云VPC私有网络 Python3 SDK方法封装]
    """
    def __init__(self, key) -> None:
        self.SecretId = key['SecretId']             # 公钥
        self.SecretKey = key['SecretKey']           # 私钥
        self.endpoint = key['VPC']['endpoint']      # 接口地址
        self.Region = key['Region']                 # 地域
        # VPC
        self.VpcId = key['VPC']['VpcId']                                        # VPC id
        self.SubnetId = key['VPC']['SubnetId']                                  # 子网 id
        self.SecurityGroupIds = key['VPC']['SecurityGroupIds']                  # 安全组id（id）
        self.InternetMaxBandwidthOut = key['VPC']['InternetMaxBandwidthOut']    # EIP带宽值
        self.InternetChargeType = key['VPC']['InternetChargeType']              # EIP计费类型

        # 签名
        self.cred = credential.Credential(self.SecretId, self.SecretKey)
        httpProfile = HttpProfile()
        httpProfile.endpoint = self.endpoint

        clientProfile = ClientProfile()
        clientProfile.httpProfile = httpProfile
        self.client = vpc_client.VpcClient(self.cred, self.Region, clientProfile)


    def describeAddressQuota(self):
        """[查询弹性公网IP配额]https://cloud.tencent.com/document/api/215/16701
        Quota响应值解释：https://cloud.tencent.com/document/api/215/15824#Quota

        Returns:
            [json]: [json response]
        """
        try:
            req = models.DescribeAddressQuotaRequest()
            params = {

            }
            req.from_json_string(json.dumps(params))

            resp = self.client.DescribeAddressQuota(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err


    def allocateAddresses(self, AddressType='EIP', AddressCount=1):
        """[创建弹性公网IP]https://cloud.tencent.com/document/api/215/16699

        Args:
            AddressType (str, optional): [EIP类型：'EIP'、'AnycastEIP'：加速IP、'HighQualityEIP'：精品IP]. 默认值：EIP.
            AddressCount (int, optional): [EIP数量]. 默认值：1.
        
        Returns:
            [json]: [json response]
        """
        try:
            req = models.AllocateAddressesRequest()
            params = {
                "AddressCount": AddressCount,
                "InternetChargeType": self.InternetChargeType,
                "InternetMaxBandwidthOut": self.InternetMaxBandwidthOut,
                "AddressType": AddressType
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.AllocateAddresses(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err


    def releaseAddresses(self, AddressIds=[]):
        """[释放弹性公网IP]https://cloud.tencent.com/document/api/215/16705

        Args:
            AddressIds ([list]): [标识 EIP 的唯一 ID 列表。EIP 唯一 ID 形如：eip-11112222。]
        
        Returns:
            [json]: [json response]
        """
        try:
            req = models.ReleaseAddressesRequest()
            params = {
                "AddressIds": AddressIds,
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.ReleaseAddresses(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err


    def associateAddress(self, AddressId, NetworkInterfaceId, PrivateIpAddress):
        """[绑定弹性公网IP]https://cloud.tencent.com/document/api/215/16700

        Args:
            AddressId ([str]): [标识 EIP 的唯一 ID。EIP 唯一 ID 形如：eip-11112222]
            NetworkInterfaceId ([str]): [要绑定的弹性网卡 ID。 弹性网卡 ID 形如：eni-11112222。]
            PrivateIpAddress ([str]): [要绑定的内网 IP。如果指定了 NetworkInterfaceId 则也必须指定 PrivateIpAddress ，表示将 EIP 绑定到指定弹性网卡的指定内网 IP 上。同时要确保指定的 PrivateIpAddress 是指定的 NetworkInterfaceId 上的一个内网 IP。]
        
        Returns:
            [json]: [json response]
        """
        try:
            req = models.AssociateAddressRequest()
            params = {
                "AddressId": AddressId,
                "NetworkInterfaceId": NetworkInterfaceId,
                "PrivateIpAddress": PrivateIpAddress
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.AssociateAddress(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err


    def disassociateAddress(self, AddressId):
        """[解绑定弹性公网IP]https://cloud.tencent.com/document/api/215/16703

        Args:
            AddressId ([str]): [标识 EIP 的唯一 ID。EIP 唯一 ID 形如：eip-11112222]

        Returns:
            [json]: [json response]
        """
        try:
            req = models.DisassociateAddressRequest()
            params = {
                "AddressId": AddressId
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.DisassociateAddress(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err


    def transformAddress(self, InstanceId):
        """[普通IP转弹性IP]https://cloud.tencent.com/document/api/215/16706

        Args:
            InstanceId ([str]): [待操作有普通公网 IP 的实例 ID]
        
        Returns:
            [json]: [json response]
        """
        try:
            req = models.TransformAddressRequest()
            params = {
                "InstanceId": InstanceId
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.TransformAddress(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def describeAddresses(self, address_id='', address_ip='', address_status='', instance_id='', network_interface_id=''):
        """[查询弹性公网IP列表]https://cloud.tencent.com/document/api/215/16702

        有且只能选择以下一个条件进行过滤查询。
        Args:
            address_id (str, optional): [按照 EIP 的唯一 ID 过滤]. Defaults to ''.
            address_ip (str, optional): [按照 EIP 的 IP 地址过滤]. Defaults to ''.
            address_status (str, optional): [按照 EIP 的状态过滤，状态包含：'CREATING'，'BINDING'，'BIND'，'UNBINDING'，'UNBIND'，'OFFLINING'，'BIND_ENI'。]. Defaults to ''.
            instance_id (str, optional): [按照 EIP 绑定的实例 ID 过滤]. Defaults to ''.
            network_interface_id (str, optional): [按照 EIP 绑定的弹性网卡 ID 过滤]. Defaults to ''.

        Returns:
            [json]: [json response]
        """

        try:
            req = models.DescribeAddressesRequest()
            if len(address_id) > 0:
                params = {"Filters": [{"Name": "address-id","Values": [ address_id ]}]}
            elif len(address_ip) > 0:
                params = {"Filters": [{"Name": "address-ip","Values": [ address_ip ]}]}
            elif len(address_status) > 0:
                params = {"Filters": [{"Name": "address-status","Values": [ address_status ]}]}
            elif len(instance_id) > 0:
                params = {"Filters": [{"Name": "instance-id","Values": [ instance_id ]}]}
            elif len(network_interface_id) > 0:
                params = {"Filters": [{"Name": "network-interface-id","Values": [ network_interface_id ]}]}

            req.from_json_string(json.dumps(params))

            resp = self.client.DescribeAddresses(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def createNetworkInterface(self, NetworkInterfaceName='eth1'):
        """[创建弹性网卡]https://cloud.tencent.com/document/api/215/15818

        Args:
            NetworkInterfaceName (str, optional): [弹性网卡名称]. 默认值：'eth1'.

        Returns:
            [json]: [json response]
        """
        try:
            req = models.CreateNetworkInterfaceRequest()
            params = {
                "VpcId": self.VpcId,
                "NetworkInterfaceName": NetworkInterfaceName,
                "SubnetId": self.SubnetId,
                "SecurityGroupIds": self.SecurityGroupIds,
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.CreateNetworkInterface(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def createAndAttachNetworkInterface(self, InstanceId, NetworkInterfaceName='eth1'):
        """[创建弹性网卡并绑定云服务器]https://cloud.tencent.com/document/api/215/43370

        Args:
            InstanceId ([str]): [云服务器实例ID]
            NetworkInterfaceName (str, optional): [弹性网卡名称，最大长度不能超过60个字节。]. 默认值'eth1'.

        Returns:
            [json]: [json response]
        """
        try:
            req = models.CreateAndAttachNetworkInterfaceRequest()
            params = {
                "VpcId": self.VpcId,
                "NetworkInterfaceName": NetworkInterfaceName,
                "SubnetId": self.SubnetId,
                "SecurityGroupIds": self.SecurityGroupIds,
                "InstanceId": InstanceId
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.CreateAndAttachNetworkInterface(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def deleteNetworkInterface(self, NetworkInterfaceId):
        """[删除弹性网卡]https://cloud.tencent.com/document/api/215/15822

        Args:
            NetworkInterfaceId ([str]): [弹性网卡实例ID，例如：eni-m6dyj72l。]
        
        Returns:
            [json]: [json response]
        """
        try:
            req = models.DeleteNetworkInterfaceRequest()
            params = {
                "NetworkInterfaceId": NetworkInterfaceId
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.DeleteNetworkInterface(req)
            # print(resp.to_json_string())
            return resp.to_json_string()
            
        except TencentCloudSDKException as err:
            # print(err)
            return err



    def attachNetworkInterface(self, NetworkInterfaceId, InstanceId):
        """[弹性网卡绑定云服务器]https://cloud.tencent.com/document/api/215/15819

        Args:
            NetworkInterfaceId ([str]): [弹性网卡实例ID，例如：eni-m6dyj72l。]
            InstanceId ([str]): [CVM实例ID。形如：ins-r8hr2upy。]

        Returns:
            [json]: [json response]
        """
        try:
            req = models.AttachNetworkInterfaceRequest()
            params = {
                "NetworkInterfaceId": NetworkInterfaceId,
                "InstanceId": InstanceId
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.AttachNetworkInterface(req)
            # print(resp.to_json_string())
            return resp.to_json_string()
            
        except TencentCloudSDKException as err:
            # print(err)
            return err


    def describeNetworkInterfaces(self, network_interface_id='', attachment_instance_id='', network_interface_name=''):
        """[查询弹性网卡列表]https://cloud.tencent.com/document/api/215/15817
        
        Args:
            network_interface_id (str, optional): [弹性网卡实例ID]. Defaults to ''.
            attachment_instance_id (str, optional): [绑定的云服务器实例ID]. Defaults to ''.
            network_interface_name (str, optional): [网卡实例名称]. Defaults to ''.
        
        注意：network_interface_id 只能单独使用，attachment_instance_id 和 network_interface_name 可以组合使用。
            
        Returns:
            [json]: [json response]
        """
        try:
            req = models.DescribeNetworkInterfacesRequest()
            if len(network_interface_id) > 0:
                params = {"Filters": [{"Name": "network-interface-id","Values": [ network_interface_id, ]}]}
            elif len(attachment_instance_id) > 0 and len(network_interface_name) > 0:
                params = {"Filters": [{"Name": "attachment.instance-id","Values": [ attachment_instance_id, ]},{"Name": "network-interface-name","Values": [ network_interface_name, ]}]}
            elif len(attachment_instance_id) > 0:
                params = {"Filters": [{"Name": "attachment.instance-id","Values": [ attachment_instance_id, ]}]}
            elif len(network_interface_name) > 0:
                params = {"Filters": [{"Name": "network-interface-name","Values": [ network_interface_name, ]}]}
        
            req.from_json_string(json.dumps(params))

            resp = self.client.DescribeNetworkInterfaces(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err


    def describeTaskResult(self, TaskId):
        """[查询异步任务执行结果]https://cloud.tencent.com/document/api/215/36271

        Args:
            TaskId ([str]): [异步任务ID]

        Returns:
            [json]: [json response]
        """
        try:
            req = models.DescribeTaskResultRequest()
            params = {
                "TaskId": TaskId
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.DescribeTaskResult(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



import time
from functools import wraps

# Write by lyc at 2020.9.4

def wrapper_retry(retry_count=3, sleep_second=3):
    '''
    函数装饰器：异常捕获，间隔重试
    retry_count: 重试次数，默认值 3
    sleep_second: 重试间隔，默认值 3（单位秒）
    '''
    def wrapper_outer_retry(func):
        @wraps(func)
        def wrapper_inner_retry(*args, **kwargs):
            count = 1   # 重试次数
            while count <= retry_count:     # 若接口返回异常，间隔3s重试3次
                try:        # 异常捕获
                    ret = func(*args, **kwargs)
                    if ret == False:    # 接口返回值是False，也认为是异常，sleep后continue
                        count += 1
                        time.sleep(sleep_second)
                        continue
                    else:
                        break       # 没有发生异常，break跳出重试循环
                except Exception as err:  # 捕获到程序异常，sleep后继续下一循环
                    print(err)
                    count += 1
                    time.sleep(sleep_second)

            return ret
        return wrapper_inner_retry
    return wrapper_outer_retry



import os
import paramiko

# Write by lyc at 2021-10-14
# 参考文档《Python paramiko 模块详解与SSH主要功能模拟》https://blog.csdn.net/u014028063/article/details/81197431

class SSH_Tools():
    """[ssh简单方法封装]
    """
    def __init__(self, host, username='root', port=22, password='', key='') -> None:
        """[初始化对象]

        Args:
            host ([str]): [主机ip]
            port (int, optional): [端口号].默认值 '22'
            username (str, optional): [用户名]. 默认root用户
            password ([str, optional]): [密码].
            ssh_key ([str, optional]): [ssh-key 私钥文件，绝对路径]
        """
        self.host = host
        self.port = port
        self.username = username
        self.password = password
        self.key = key
    

    def connect(self, auth_type='sshclient'):
        """[sshclient 方式登录]

        Args:
            auth_type ([str, optional]): 'sshclient' 或 'transport'

        Returns:
            [obj]: [SSHClient的对象]
        """
        try:
            ssh = paramiko.SSHClient()      # 创建ssh对象
            ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())       # 允许连接不在know_hosts文件中的主机

            # sshclient 方式
            if auth_type == 'sshclient':
                # 基于用户名和密码
                if len(self.password) > 0:
                    ssh.connect(hostname=self.host, port=self.port, username=self.username, password=self.password, timeout=5)
                
                # ssh-key 秘钥方式
                elif len(self.key) > 0 and os.path.exists(self.key):
                    key = paramiko.RSAKey.from_private_key(self.key)
                    ssh.connect(hostname=self.host, port=self.port, username=self.username, pkey=key, timeout=5)

                else:
                    # 密码或密钥有误，主动抛出异常
                    raise Exception("Please check password or key.")
            
            # transport 方式
            # elif auth_type == 'transport':
            #     transport = paramiko.Transport((self.host, self.port))      # 实例化一个transport对象
               
            #     # 基于用户名和密码
            #     if len(self.password) > 0:
            #         transport.connect(username=self.username, password=self.password)       # 建立连接

            #         # 将sshclient的对象的transport指定为以上的transport
            #         ssh._transport = transport
            #         # 关闭连接
            #         # transport.close()
            
 
            return ssh      # 返回已经连接成功的ssh对象

        except Exception as err:
            print(err)



    def ssh_exec_command(self, command, open_log=True):
        """[ssh远程linux主机执行命令]

        Args:
            command ([str]): [Linux shell命令，多条命令使用 ';' 分隔符分割。]
            open_log (bool, optional): ['True'开启采集命令 stdout和stderr；'False' 关闭采集。]. Defaults to True.

        Returns:
            [dict]: [返回字典：'stdout'命令正确结果，'stderr'命令失败报错]
        """
        ssh = self.connect()    # ss连接目标主机
        try:
            result = {'stdout': '', 'stderr': '', 'open_log': open_log}
            stdin, stdout, stderr = ssh.exec_command(command)
            if open_log:
                result['stdout'] = (stdout.read()).decode('utf-8')
                result['stderr'] = (stderr.read()).decode('utf-8')
            ssh.close()
            return result        # 返回命令结果

        except Exception as err:
            print(err)



    def push_file(self, local_file, remote_file):
        """[sftp上传文件到目标主机]

        Args:
            local_file ([str]): [本地文件绝对路径]
            remote_file ([str]): [目标主机文件绝对路径]

        Returns:
            [str]: ['0'上传成功]
        """
        ssh = self.connect()
        try:
            sftp = ssh.open_sftp()      # 创建sftp对象
            sftp.put(local_file, remote_file)
            sftp.close()
            ssh.close()
            return '0'

        except Exception as err:
            print(err)
            return err





import json
from tencentcloud.common import credential
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
from tencentcloud.cvm.v20170312 import cvm_client, models

# Write by lyc at 2021-10-9

class CVM():
    """[腾讯云CVM云主机 Python3 SDK方法封装]
    """
    def __init__(self, key) -> None:
        self.SecretId = key['SecretId']             # 公钥
        self.SecretKey = key['SecretKey']           # 私钥
        self.endpoint = key['CVM']['endpoint']      # 接口地址
        self.Region = key['Region']                 # 地域
        self.Zone = key['Zone']                     # 可用区
        # CVM
        self.Password = key['CVM']['Password']                      # 实例密码
        self.InstanceName = key['CVM']['InstanceName']              # 实例名称
        self.InstanceType = key['CVM']['InstanceType']              # 机型
        self.ImageId = key['CVM']['ImageId']                        # 镜像id
        self.InstanceChargeType = key['CVM']['InstanceChargeType']  # 实例计费类型
        # VPC
        self.VpcId = key['VPC']['VpcId']                                        # VPC id
        self.SubnetId = key['VPC']['SubnetId']                                  # 子网 id
        self.InternetMaxBandwidthOut = key['VPC']['InternetMaxBandwidthOut']    # EIP带宽值
        self.SecurityGroupIds = key['VPC']['SecurityGroupIds']                  # 安全组id（列表）
        self.InternetChargeType = key['VPC']['InternetChargeType']              # EIP计费类型
        # 签名
        self.cred = credential.Credential(self.SecretId, self.SecretKey)
        httpProfile = HttpProfile()
        httpProfile.endpoint = self.endpoint

        clientProfile = ClientProfile()
        clientProfile.httpProfile = httpProfile
        self.client = cvm_client.CvmClient(self.cred, self.Region, clientProfile)


    def describeAccountQuota(self):
        """[查询用户配额详情]https://cloud.tencent.com/document/api/213/55628

        Returns:
            [json]: [json response]
        """
        try:
            req = models.DescribeAccountQuotaRequest()
            params = {
                "Filters": [
                    {
                        "Name": "zone",
                        "Values": [ self.Zone, ],
                    }
                ]
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.DescribeAccountQuota(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def describeZones(self):
        """[查询可用区列表]https://cloud.tencent.com/document/api/213/15707

        Returns:
            [json]: [json response]
        """
        try:
            req = models.DescribeZonesRequest()
            params = {

            }
            req.from_json_string(json.dumps(params))

            resp = self.client.DescribeZones(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def describeInstanceTypeConfigs(self, zone, instance_family):
        """[查询实例机型列表]https://cloud.tencent.com/document/api/213/15749

        Args:
            zone ([str]): [按照【可用区】进行过滤。可用区形如：ap-guangzhou-1。]

        Returns:
            [json]: [json response]
        """
        try:
            req = models.DescribeInstanceTypeConfigsRequest()
            params = {
                "Filters": [
                    {
                        "Name": "zone",
                        "Values": [ zone, ]
                    },
                    {
                        "Name": "instance-family",
                        "Values": [ instance_family, ],
                    }
                ]
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.DescribeInstanceTypeConfigs(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def runInstances(self, InstanceCount=1, Period=1, RenewFlag='NOTIFY_AND_AUTO_RENEW'):
        """[创建实例]https://cloud.tencent.com/document/api/213/15730

        Args:
            InstanceCount (int, optional): [购买实例数量。包年包月实例取值范围：[1，300]，按量计费实例取值范围：[1，100]。]. 默认取值：1                      
            Period (int, optional): [购买实例的时长，单位：月。取值范围：1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 24, 36, 48, 60。]. Defaults to 1.
            RenewFlag (str, optional): [自动续费标识]. 默认值 'NOTIFY_AND_AUTO_RENEW'通知过期且自动续费；'NOTIFY_AND_MANUAL_RENEW' 通知过期不自动续费
        
        Returns:
            [json]: [json response]
        """
        try:
            req = models.RunInstancesRequest()

            if self.InstanceChargeType == 'POSTPAID_BY_HOUR':       # 按时计费
                params = {
                    "InstanceChargeType": self.InstanceChargeType,
                    "Placement": {
                        "Zone": self.Zone
                    },
                    "InstanceType": self.InstanceType,
                    "ImageId": self.ImageId,
                    "VirtualPrivateCloud": {
                        "VpcId": self.VpcId,
                        "SubnetId": self.SubnetId
                    },
                    "InternetAccessible": {
                        "InternetChargeType": self.InternetChargeType,
                        "InternetMaxBandwidthOut": self.InternetMaxBandwidthOut,
                        "PublicIpAssigned": True
                    },
                    "InstanceCount": InstanceCount,
                    "InstanceName": self.InstanceName,
                    "LoginSettings": {
                        "Password": self.Password
                    },
                    "SecurityGroupIds": self.SecurityGroupIds
                }
            elif self.InstanceChargeType == 'PREPAID':              # 包月计费
                params = {
                    "InstanceChargeType": self.InstanceChargeType,
                    "InstanceChargePrepaid": {
                        "Period": Period,
                        "RenewFlag": RenewFlag
                    },
                    "Placement": {
                        "Zone": self.Zone
                    },
                    "InstanceType": self.InstanceType,
                    "ImageId": self.ImageId,
                    "VirtualPrivateCloud": {
                        "VpcId": self.VpcId,
                        "SubnetId": self.SubnetId
                    },
                    "InternetAccessible": {
                        "InternetChargeType": self.InternetChargeType,
                        "InternetMaxBandwidthOut": self.InternetMaxBandwidthOut,
                        "PublicIpAssigned": True
                    },
                    "InstanceCount": InstanceCount,
                    "InstanceName": self.InstanceName,
                    "LoginSettings": {
                        "Password": self.Password
                    },
                    "SecurityGroupIds": self.SecurityGroupIds
                }

            req.from_json_string(json.dumps(params))

            resp = self.client.RunInstances(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def stopInstances(self, InstanceIds=[], ForceStop=True):
        """[关闭实例]https://cloud.tencent.com/document/api/213/15743

        Args:
            InstanceIds (list): [实例ID]. 
            ForceStop (bool, optional): [是否在正常关闭失败后选择强制关闭实例
                                            TRUE：表示在正常关闭失败后进行强制关闭
                                            FALSE：表示在正常关闭失败后不进行强制关闭]. Defaults to True.
        
        Returns:
            [json]: [json response]
        """
        try:
            req = models.StopInstancesRequest()
            params = {
                "InstanceIds": InstanceIds,
                "ForceStop": ForceStop
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.StopInstances(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def terminateInstances(self, InstanceIds=[]):
        """[退还实例]https://cloud.tencent.com/document/api/213/15723

        Args:
            InstanceIds (list, optional): [实例ID]. 

        Returns:
            [json]: [json response]
        """
        try:
            req = models.TerminateInstancesRequest()
            params = {
                "InstanceIds": InstanceIds
            }
            req.from_json_string(json.dumps(params))

            resp = self.client.TerminateInstances(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err



    def describeInstances(self, instance_id='', instance_state='', public_ip_address=''):
        """[查看实例列表]https://cloud.tencent.com/document/api/213/15728

        注意：以下参数有且只能选择一个进行过滤查询。
        Args:
            instance_id (str, optional): [按照【实例ID】进行过滤]. Defaults to ''.
            instance_state (str, optional): [按照【实例状态】进行过滤，实例状态。取值范围：
                                            PENDING：表示创建中
                                            LAUNCH_FAILED：表示创建失败
                                            RUNNING：表示运行中
                                            STOPPED：表示关机
                                            STARTING：表示开机中
                                            STOPPING：表示关机中
                                            REBOOTING：表示重启中
                                            SHUTDOWN：表示停止待销毁
                                            TERMINATING：表示销毁中。].Defaults to ''.
            public_ip_address (str, optional): [按照【实例主网卡的公网IP】进行过滤，包含实例创建时自动分配的IP和实例创建后手动绑定的弹性IP。]. Defaults to ''.

        Returns:
            [json]: [json response]
        """
        try:
            req = models.DescribeInstancesRequest()

            if len(instance_id) > 0:
                params = {"Filters": [{"Name": "instance-id", "Values": [ instance_id, ]}]}
            elif len(instance_state) > 0:
                params = {"Filters": [{"Name": "instance-state", "Values": [ instance_state, ]}]}
            elif len(public_ip_address) > 0:
                params = {"Filters": [{"Name": "public-ip-address", "Values": [ public_ip_address, ]}]}

            req.from_json_string(json.dumps(params))

            resp = self.client.DescribeInstances(req)
            # print(resp.to_json_string())
            return resp.to_json_string()

        except TencentCloudSDKException as err:
            # print(err)
            return err


